# Keys that are expected to be in diagnostics report.
BALENA_DEVICE_ID_KEY = 'BALENA_DEVICE_UUID'
BALENA_NAME_KEY = 'BALENA_DEVICE_NAME_AT_INIT'
BLUETOOTH_KEY = 'bluetooth'
DEVICE_STATUS_KEY = 'device_status'
DISK_IMAGE_KEY = 'disk_image'
ECC_KEY = 'ECC'
ERRORS_KEY = 'errors'
ETH_MAC_ADDRESS_KEY = 'eth_mac_address'
FIRMWARE_SHORT_HASH_KEY = 'firmware_short_hash'
FIRMWARE_VERSION_KEY = 'FIRMWARE_VERSION'
FREQUENCY_KEY = 'FREQ'
LORA_KEY = 'lora'
LTE_KEY = 'lte'
ONBOARDING_KEY = 'onboarding_key'
PF_KEY = 'legacy_pass_fail'
LORA_KEY = 'lora'
PUBLIC_KEY = 'public_key'
SERIAL_NUMBER_KEY = 'serial_number'
VARIANT_KEY = 'VARIANT'
WIFI_MAC_ADDRESS_KEY = 'wifi_mac_address'

# gatewayrs diagnostic keys
VALIDATOR_ADDRESS_KEY = 'validator_address'
VALIDATOR_URI_KEY = 'validator_uri'
VALIDATOR_BLOCK_HEIGHT_KEY = 'validator_height'
VALIDATOR_BLOCK_HEIGHT_SHORT_KEY = 'MH'
VALIDATOR_BLOCK_AGE = 'validator_block_age'
GATEWAY_PUBKEY_KEY = 'gateway_pubkey'
GATEWAY_REGION_KEY = 'gateway_region'
GATEWAY_REGION_SHORT_KEY = 'RE'
